<!DOCTYPE html>
<html lang="en">
<head>
    <title>{{ name }} package</title>
</head>
<body>
<h2>{{ name }} links</h2>
{% for version in versions %}
    <a href="{{ url_for("get_python_dist_file", pkg_name=name, version=version.version, dist_type=version.filetype,  filename=version.filename) }}{% if version.sha256_digest -%}#sha256=
        {{ version.sha256_digest }}{% elif version.md5_digest %}#md5={{ version.md5_digest }}{%- endif %}"
            {% if version.requires_python %}
       data-requires-python="{{ version.requires_python }}"
            {% endif %}
    >{{ version.filename }}</a><br/>
{% endfor %}
{% if readme %}
    <style>
        .code {
            background-color: #eee;
            border-radius: 3px;
            font-family: courier, monospace;
            padding: 0 0;
            margin: 0 0;
        }
    </style>
    <h2>{{ name }} readme</h2>
    <div class="code">
        <pre>
{{ readme }}
        </pre>
    </div>
    <br>
{% endif %}
</body>
</html>